#!/usr/bin/python
"""
========================================================================
plot_4z_hrp_packet
========================================================================
DCN:     IEEE P802.15-20-0002-00-004z
Project: IEEE P802.15 Working Group for
         Wireless Personal Area Networks (WPANs)
Title:   15.4z HRP UWB PHY Test Vectors
Date:    3 January, 2020
Source:  Manuel Lafer (NXP Semiconductors)
         Frank Leong (NXP Semiconductors)

Notice:  This document has been prepared to assist the IEEE P802.15.
         It is offered as a basis for discussion and is not binding on
         the contributing individual(s) or organization(s). The
         material in this document is subject to change in form and
         content after further study. The contributor(s) reserve(s) the
         right to add, amend or withdraw material contained herein.
Release: The contributor(s) acknowledge(s) and accept(s) that this
         contribution becomes the property of IEEE and may be made
         publicly available by P802.15.


========================================================================
Description
========================================================================
Script to plot pulses from an IEEE 802.15.4z HRP UWB PHY test vector
MAT file, passed via command-line argument (parameter).

Dependencies:
- python v3.7.3 (developed using Anaconda3-2019.07-Windows-x86_64)
- argparse
- decimal
- numpy
- scipy
- matplotlib
- commpy

Anaconda provides most dependencies. commpy can be added
by entering the following in the Anaconda prompt:
"pip install scikit-commpy"

======================================================================== 
========================================================================
"""


import argparse
import decimal
import scipy.io as sio
from scipy.signal import hilbert
import numpy as np
import matplotlib.pyplot as plt
from matplotlib.ticker import EngFormatter
from commpy.filters import rrcosfilter



def min_phase(firseq, fft_pts):
    """A function to convert a FIR impulse response to min-phase
    
    A function that calculates the minimum-phase equivalent of a FIR
    impulse response based on a Discrete Hilbert Transform
    applied in the frequency domain.
    """
    # Recommended fft_pts >= 32768
    #
    # For guidelines on the required number of FFT points, see:
    # N. Damera-Venkata, B. L. Evans and S. R. McCaslin,
    # "Design of optimal minimum-phase digital FIR filters using
    # discrete Hilbert transforms," in IEEE Transactions on Signal
    # Processing, vol. 48, no. 5, pp. 1491-1495, May 2000.
    max_phase = np.real(np.fft.ifft( \
        np.exp(hilbert(np.real(np.log(np.fft.fft( \
        firseq, fft_pts)))))))
    tmp_min_phase = max_phase[::-1] # reverse max phase
    return tmp_min_phase[0:len(firseq)] # maintain length



# Define command line program description
text = "Script to plot pulses from a test vector MAT file."

# Initiate parser with command line description
parser = argparse.ArgumentParser(description = text)

# Add long and short argument for input file
parser.add_argument("--input", "-i", \
    help = "set input data file (default is \"bprf_1.mat\")")

# Add long and short argument for carrier frequency multiplier
parser.add_argument("--carrier", "-c", \
    help = "set carrier frequency multiplier x 499.2 MHz")

# Add long and short argument for oversampling ratio
parser.add_argument("--osr", "-o", \
    help = "set oversampling ratio (default is 64)")

# Add long and short argument for minimum-phase conversion
parser.add_argument("--minphase", "-m", \
    help = "set minimum-phase pulse (default is linear-phase)", \
    action = "store_true")

# Parse arguments from the command line
args = parser.parse_args()


# Check for "--input" => Default file name is "bprf_1.mat"
if args.input:
    infilename = args.input
else:
    infilename = "bprf_1.mat"

print("Processing input data file   : %s" % infilename)


# Check for "--carrier" => Default is 16 (~8.0 GHz)
if args.carrier:
    carriermult = decimal.Decimal(args.carrier)
else:
    carriermult = decimal.Decimal(16)

carrierfreq = decimal.Decimal(carriermult * decimal.Decimal(499.2e6))

print("Using carrier multiplier     : %s" % \
    carriermult.to_eng_string())
print("Using carrier frequency [Hz] : %s" % \
    carrierfreq.normalize().to_eng_string())


# Check for "--osr" => Default is 64
# (lower than 64 or higher than 256 is generally a bad idea)
if args.osr:
    osr = decimal.Decimal(args.osr)
else:
    osr = decimal.Decimal(64)

print("Using oversampling ratio     : %s" % osr.to_eng_string())


# Set chip rate associated with the test vector
chiprate = 499.2e6

# Define oversampled linear-phase RRC reference pulse
rrcpulse_tmp = rrcosfilter(8*int(osr), 0.5, 1e-9, chiprate*int(osr))[1]


# Check for "--minphase" => Set pulse shape accordingly
if args.minphase:
    # Set FFT size to be large, ensuring accuracy
    fftsize = 65536
    # Convert reference pulse to min-phase
    rrcpulse = min_phase(rrcpulse_tmp, fftsize)
    print("Pulse shape is minimum-phase RRC")
else:
    # Keep original linear-phase reference pulse
    rrcpulse = rrcpulse_tmp
    print("Pulse shape is linear-phase RRC")


# Load test vector .mat file
mat_contents = sio.loadmat(infilename)

sig_tmp = np.squeeze(mat_contents["pulses_all"])


# # # # # # # # # # # # # # #
# # # Shape the signal  # # #
# # # # # # # # # # # # # # #

# Upsample the test vector
sig = np.zeros(int(osr)*len(sig_tmp))
sig[::int(osr)] = sig_tmp

# Convolve the test vector with the RRC reference pulse
bbmod = np.convolve(sig,rrcpulse)


# # # # # # # # # # # # # #
# # # Plot the result # # #
# # # # # # # # # # # # # #

# Define the associated time axis
timeaxis = np.arange(0,len(bbmod)/(chiprate*int(osr)), \
    1/(chiprate*int(osr)))

# Prepare plot aspect ratio
plt.figure(figsize=(16,9))

# Plot the upconverted signal on a 13*chiprate (i.e., 6.5 GHz) carrier
# plt.plot(timeaxis, bbmod*np.sin(timeaxis*2*np.pi*13*chiprate))

# Plot the upconverted signal on a 16*chiprate (i.e., 8.0 GHz) carrier
# plt.plot(timeaxis, bbmod*np.sin(timeaxis*2*np.pi*16*chiprate))

# Plot the upconverted signal on specified carrier (default 8.0 GHz)
plt.plot(timeaxis, \
    bbmod*np.sin(timeaxis*2*np.pi*float(carriermult)*chiprate))

# Uncomment next line to also plot the baseband modulation (envelope)
# plt.plot(timeaxis, bbmod)

# Set x-axis engineering format
formatter0 = EngFormatter(unit="s")
plt.gca().xaxis.set_major_formatter(formatter0)

# Add title and axis labels
plt.title("Upconverted Signal")
plt.xlabel("Time")
plt.ylabel("Amplitude [Arbitrary Units]")

# Instantiate figure
plt.show()
